/**
* \file batiment.hpp
* \brief GS::Batiment
* \author Yoha
* \version alpha-0.0.07
* \date Aot 2009
*
* Definition des batiments
*
*/

#ifndef BATIMENT_HPP
#define BATIMENT_HPP

/// Mode dynamique
#define SFML_DYNAMIC

#include <vector>
#include <SFML/Graphics.hpp>

#include "lunar.hpp"
#include "etre.hpp"
#include "gamesystem.hpp"
#include "typebatiment.hpp"
#include "typeressource.hpp"
#include "objet.hpp"

/// Game System
namespace GS
{
	/// Liste chainee des objets a fabriquer
	struct ListeElements
	{
		/// Numero de la ressource dans le batiment
		int numRessource;
		/// Numero de l'objet dans le batiment
		int numObjet;
		
		/// Quantite de ressource a fabriquer
		double quantite;
		
		/// Pointeur sur l'element suivant
		ListeElements* elementSuivant;
	};
	
	/// Represente un batiment
	class Batiment : public Etre
	{
		public:
			/// Constructeur
			Batiment(GS::Etre* parent, TiXmlElement* element = NULL, const char* scriptFile = "");
			/// \brief Constructeur Lua
			/// \param L Parametre lua
			Batiment(lua_State* L);
			/// Destructeur
			virtual ~Batiment();
			
			/// \brief Charge une sauvegarde
			/// \param element Element contenant la sauvegarde
			virtual void Charge();
			/// \brief Sauvegarde
			virtual TiXmlElement* Sauvegarde();
			
			/// \brief Methode appelee lorsque le personnage construit
			/// \brief acteur Personnage qui construit
			/// \brief tempsFrame Duree de la frame
			bool Construit(Etre* acteur, double tempsFrame);
			/// \brief Methode appelee lorsque le personnage travaille
			/// \brief fabricant Personnage qui trabaille
			/// \brief tempsFrame Duree de la frame
			void Travaille(Etre* fabricant, double tempsFrame);
			
			/// \brief Etre: Retourne le texte qui s'affiche lors du pointage au curseur
			std::string GetTexte() const;
			
			/// \brief Methode de dessin du batiment
			/// \param App Pointeur sur la fenetre de rendu
			/// \param x Abscisse de l'emplacement du batiment
			/// \param y Ordonnee de l'emplacement du batiment
			void Dessine(sf::RenderTarget* const App, const double x, const double y) const;
			
			/// \brief Ajoute une ressource a la liste de travail
			/// \param num Numero de la ressource a transformer
			/// \param quantite Quantite de ressource a transformer
			void AjouteRessourceAFabriquer(const unsigned int num, const unsigned int quantite);
			/// \brief Ajoute un objet a la liste de travail
			/// \param num Numero de l'objet a fabriquer
			void AjouteObjetAFabriquer(const unsigned int num);
			/// \brief Enleve un objet de la liste de travail
			/// \param num Numero de l'objet dans la liste de travail
			void EnleveElementListeTravail(const unsigned int num);
			
			/// \brief Retourne l'objet pris par le personnage
			/// \param num Numero de l'objet a prendre
			GS::Objet* PrendObjet(const unsigned int num);
			/// \brief Retourne le type de l'objet constructible
			/// \param num Numero du type d'objet
			GS::TypeObjet* GetTypeObjetFabricable(const unsigned int num) const;
			/// \brief Retourne le type de la ressource a tranformer
			/// \param num Numero de la ressource
			GS::TypeRessource* GetTypeRessourceATransformer(const unsigned int num) const;
			/// \brief Retourne le type de l'objet a fabriquer
			/// \param num Numero de l'objet
			GS::TypeObjet* GetTypeObjetAFabriquer(const unsigned int num) const;
			
			/// \brief Accesseur: definit le type du batiment
			/// \param type Pointeur sur un type de batiment
			void SetType(TypeBatiment* type);
			/// \brief Definit la position du batiment
			/// \param x Abscisse
			/// \param y Ordonnee
			void SetPosition(const double x, const double y);
			
			/// \brief Accesseur: renvoie le type du batiment
			GS::TypeBatiment* GetTypeBatiment() const;
			/// \brief Accesseur: retourne le booleen de l'etat de construction
			bool GetConstruit() const;
			/// \brief Accesseur: renvoie la quantite d'une ressource deja utilisee pour la construction
			/// \param num Identifiant de la ressource
			double GetQuantitesRessourceConstruction(const unsigned int num) const;
			/// \brief Accesseur: renvoie le nombre d'objets d'un type construits
			/// \param num Numero du type d'objet
			unsigned int GetNombreObjetsConstruits(const unsigned int num) const;
			/// \brief Accesseur: renvoie l'avancement de fabrication d'un objet
			/// \param num Numero du type d'objet
			double GetAvancementFabricationObjet(const unsigned int num) const;
			/// \brief Accesseur: renvoie la liste des objets en attente de construction
			ListeElements* GetListeElements() const;
			
			/// \brief Retourne la possibilite de construire un batiment terrestre dans ce rectangle
			/// \brief x Abcisse du batiment
			/// \brief y Ordonnee du batiment
			/// \brief largeur Largeur du batiment
			/// \brief hauteur Hauteur du batiment
			bool GetConstructible(const double x, const double y, const unsigned int largeur, const unsigned int hauteur) const;
		
		private:
			/// Permet l'utilisation de la classe par Lunar
			friend class Lunar<Batiment>;
			/// Nom de la classe Lunar
			static const char className[];
			/// Descripteurs de methodes Lunar
			static Lunar<Batiment>::RegType methods[];
			
			/// Pointeur sur le script Lua de l'IA
			lua_State* mIA;
			
			/// Pointeur sur le type du batiment
			TypeBatiment* mTypeBatiment;
			
			/// La batiment a-t-il ete construit
			bool mConstruit;
			/// Vie actuelle du batiment
			double mVie;
			
			/// Quantites de ressources deja utilisees pour la construction
			std::vector <double> mQuantitesRessourcesConstruction;
			
			/// Avancements de la fabrication des objets
			std::vector <double> mAvancementsFabricationObjet;
			/// Nombres d'objets construits
			std::vector <unsigned int> mNombresObjetsFabriques;
			
			/// Liste de travail
			struct ListeElements* mListeElementsAFabriquer;
	};
}

#endif
