/**
* \file monde.hpp
* \brief GS::Monde
* \author Yoha
* \version alpha-0.0.07
* \date Aot 2009
*
* Definition du monde
*
*/

#ifndef MONDE_HPP
#define MONDE_HPP

/// Mode dynamique
#define SFML_DYNAMIC

#include <SFML/Graphics.hpp>
#include "tinyxml/tinyxml.h"

#include "terressource.hpp"
#include "typebatiment.hpp"
#include "batiment.hpp"
#include "personnage.hpp"

/// Game System
namespace GS
{
	/// Represente la carte
	class Monde : public Etre
	{
		public:
			/// \brief Constructeur
			/// \param l Largeur en terrains
			/// \param h Hauteur en terrains
			/// \param scriptFile Fichier de l'IA
			Monde(GS::Etre* parent, TiXmlElement* element = NULL);
			/// \brief Destructeur
			virtual ~Monde();
			
			/// \brief Charge une sauvegarde
			/// \param element Element contenant la sauvegarde
			void Charge();
			/// \brief Sauvegarde
			TiXmlElement* Sauvegarde();
			
			/// \brief Permet au monde de choisir le personnage joueur
			/// \param personnage Reference sur un pointeur de personnage
			void SetPersonnageJoueur(GS::Personnage** personnage);
			
			/// \brief Methode de dessin de la carte
			/// \param App Pointeur sur la fenetre de rendu
			void Dessine(sf::RenderTarget* const App);
			/// \brief Deplace le point de vue sur la carte
			/// \param x Abscisse
			/// \param y Ordonnee
			void SetVue(const double x, const double y);
			/// \brief Deroulement du round
			/// \param tempsFrame Duree de la frame
			void DerouleRound(float tempsFrame);
			
			/// \brief Retourne l'abscisse du premier pixel de la carte
			double GetImgX() const;
			/// \brief Retourne l'ordonnee du premier pixel de la carte
			double GetImgY() const;
			
			/// \brief Retourne l'eventuel etre vise
			/// \param x Abscisse du point vise
			/// \param y Ordonnee du point vise
			GS::Etre* GetEtrePoint(const double x, const double y);
			/// \brief Retourne l'ID du type de terrain vise
			/// \param x Abscisse du point vise
			/// \param y Ordonnee du point vise
			unsigned int GetIdTerrain(const double x, const double y) const;
			
			/// \brief Ajoute un etre a la liste
			/// \brief etre Pointeur sur l'etre
			void AjouteEtre(GS::Etre* etre);
			/// \brief Retourne un pointeur sur l'etre demande
			/// \param num Identifiant de l'etre
			GS::Etre* GetEtre(const unsigned int num);
			
			/// \brief Cherche une ressource a une distance maximale donnee d'un point, retourne NULL si non trouve
			/// \param x Abscisse du point de recherche
			/// \param y Ordonnee du point de recherche
			/// \param distanceMax Distance maximale de recherche
			/// \param type Type de la ressource a chercher
			GS::TerRessource* ChercheTerRessource(const double x, const double y, GS::TypeTerRessource* type) const;
			
			/// \brief x Abcisse du batiment
			/// \brief y Ordonnee du batiment
			/// \brief largeur Largeur du batiment
			/// \brief hauteur Hauteur du batiment
			bool GetConstructibleAbsolu(const double x, const double y, const unsigned int largeur, const unsigned int hauteur) const;
			/// \brief Retourne la possibilite de placer une ressource sur ce rectangle
			/// \brief x Abcisse de la ressource
			/// \brief y Ordonnee de la ressource
			/// \brief largeur Largeur de la ressource
			/// \brief hauteur Hauteur de la ressource
			bool GetPeutPlacerRessource(const double x, const double y, const double largeur, const double hauteur) const;
			/// \brief Retourne la possibilite de construire un batiment terrestre dans ce rectangle
			/// \brief x Abcisse du batiment
			/// \brief y Ordonnee du batiment
			/// \brief largeur Largeur du batiment
			/// \brief hauteur Hauteur du batiment
			bool GetConstructible(const double x, const double y, const unsigned int largeur, const unsigned int hauteur) const;
			/// \brief Construit un batiment a l'emplacement indique si possible
			/// \brief x Abcisse du batiment
			/// \brief y Ordonnee du batiment
			/// \brief batimentAConstruire Pointeur sur le batiment a construire
			GS::Batiment* ConstruitBatiment(const double x, const double y, GS::TypeBatiment* batimentAConstruire);
			
			/// \brief Creer un objet
			GS::Objet* CreerObjet();
			
		private:
			/// Abscisse du point de vision
			double mXvue;
			/// Ordonnee du point de vision
			double mYvue;
			
			/// Abscisse du Monde sur l'cran
			double mXSprite;
			/// Ordonnee du Monde sur l'cran
			double mYSprite;
			
			/// Largeur
			unsigned int mLargeur;
			/// Hauteur
			unsigned int mHauteur;
			/// Nombre de cases que comporte la grille des terrains
			unsigned int mNombreCasesTerrains;
			
			/// Grilles des terrains
			unsigned int* mGrilleTerrains;
			/// Image pour l'affichage des terrains
			sf::Image* mImage;
			/// Afficheur de l'image
			sf::Sprite* mSpriteImage;
			
			/// Liste des ressources de terrain
			std::vector <GS::Etre*> mEtres;
			/// Nombre de ressources de terrain
			unsigned int mNombreEtres;
			
			/// Liste des ressources de terrain
			std::vector <GS::TerRessource*> mTerRessources;
			/// Nombre de ressources de terrain
			unsigned int mNombreTerRessources;
			
			/// Tableau des personnages
			std::vector <GS::Personnage*> mPersonnages;
			/// Nombre de personnages
			unsigned int mNombrePersonnages;
			/// Reference sur le pointeur du personnage joueur
			GS::Personnage** mPersonnageJoueur;
			
			/// Tableau des batiments
			std::vector <GS::Batiment*> mBatiments;
			/// Nombre de ressources de terrain
			unsigned int mNombreBatiments;
			
			/// Tableau des objets
			std::vector <GS::Objet*> mObjets;
			/// Nombre de objets
			unsigned int mNombreObjets;
	};
}

#endif
