/**
* \file personnage.hpp
* \brief GS::Personnage
* \author Yoha
* \version alpha-0.0.07
* \date Aot 2009
*
* Definition de la classe Personnage
*
*/

#ifndef PERSONNAGE_HPP
#define PERSONNAGE_HPP

/// Mode dynamique
#define SFML_DYNAMIC

#include <iostream>
#include <SFML/Graphics.hpp>

#include "lunar.hpp"
#include "etre.hpp"
#include "gamesystem.hpp"
#include "texte.hpp"
#include "terressource.hpp"
#include "objet.hpp"
#include "batiment.hpp"

namespace GS
{
	/// Cette classe represente un personnage, joueur ou non
	class Personnage : public ReceveurChaine, public Etre
	{
		public:
			/// \brief Constructeur
			/// \param map Pointeur sur le monde sur lequel se trouve le personnage
			/// \param interface Pointeur sur l'interface (par l'intermediaire d'un etre)
			/// \param scriptFile Fichier de l'IA
			Personnage(GS::Etre* parent, TiXmlElement* element = NULL, const char* scriptFile = "");
			/// \brief Constructeur Lua
			/// \param L Parametre lua
			Personnage(lua_State* L);
			/// \brief Destructeur
			virtual ~Personnage();
			
			/// \brief Charge une sauvegarde
			/// \param element Element contenant la sauvegarde
			void Charge();
			/// \brief Sauvegarde
			TiXmlElement* Sauvegarde();
			
			/// \brief Capture des clics de la souris
			/// \param x Abscisse du clic
			/// \param y Ordonnee du clic
			bool Catch(const double x, const double y);
			
			/// \brief ReceveurChaine: Traite la reception d'une phrase
			/// \param txt ReceveurChaine: La phrase transmise
			void RecoitChaine(const std::string txt);
			
			/// \brief Deplace le personnage au clavier
			/// \param tempsFrame Estimation du temps que dure une frame
			/// \param haut Touche directionnelle Haut pressee
			/// \param droite Touche directionnelle Droite pressee
			/// \param bas Touche directionnelle Bas pressee
			/// \param gauche Touche directionnelle Gauche pressee
			void DerouleRound(const float tempsFrame);
			/// \brief Methode d'affichage du personnage
			/// \param App Pointeur sur la fenetre de rendu
			/// \param central Le personnage est au centre de l'ecran
			void Dessine(sf::RenderTarget* const App, const double x, const double y) const;
			
			/// \brief Envoie le personnage vers un objet
			/// \param cible Etre cible par le personnage
			void Go(Etre* cible);
			/// \brief Envoie le personnage a un point de la carte
			/// \param x Abscisse du point
			/// \param y Ordonnee du point
			void Go(const double x, const double y);
			/// \brief Construit un batiment a l'emplacement indique si possible
			/// \brief x Abcisse du batiment
			/// \brief y Ordonnee du batiment
			/// \brief batimentAConstruire Pointeur sur le batiment a construire
			GS::Batiment* ConstruitBatiment(const double x, const double y, GS::TypeBatiment* batimentAConstruire);
			
			/// \brief Retourne le nom du personnage
			std::string GetNom() const;
			/// \brief Retourne un pointeur sur le batiment actuel
			GS::Batiment* GetBatimentActuel() const;
			/// \brief Retourne le texte a afficher pour decrire le type de batiment
			/// \param typeBatiment Type de batiment dont la description doit etre faite
			std::string GetTexteTypeBatiment(const GS::TypeBatiment* typeBatiment) const;
			/// \brief Retourne le texte a afficher lors du survol d'un etre
			/// \param etre Etre a decrire
			std::string GetTexteEtre(GS::Etre* etre) const;
			
			/// \brief Permet de connaitre la quantite de ressources possedee
			/// \param idRessource ID de la ressource
			double GetQuantiteRessource(const unsigned int idRessource) const;
			/// \brief Ajoute des ressources au personnage
			/// \param idRessource ID de la ressource
			/// \param quantite Quantite de ressource
			double AjouteRessource(const unsigned int idRessource, const double quantite);
			/// \brief Eleve des ressources au personnage
			/// \param idRessource ID de la ressource
			/// \param quantite Quantite de ressource
			double EnleveRessource(const unsigned int idRessource, const double quantite);
			
			/// \brief Accesseur: renvoie un pointeur sur le vector de listes chainees d'objets transportes
			std::vector < std::vector < std::vector < std::vector <GS::Objet*> > >	>* GetObjetsTransportes();
			/// \brief Ajoute un objet a l'inventaire du personnage
			bool AjouteObjet(Objet* objet);
			
			/// \brief Retourne le monde dans lequel se trouve le personnage
			/// \param L Parametre lua
			int GetMonde(lua_State* L);
			/// \brief Fait parler le personnage
			/// \param L Parametre lua
			int Dire(lua_State* L);
			/// \brief Deplace le personnage (Lua)
			/// \param L Parametre lua
			int Aller(lua_State* L);
			/// \brief Envoie le personnage a une ressource du type demande, si une telle ressource est trouvee
			/// \param L Parametre lua
			int GoRessource(lua_State* L);
			
		private:
			/// Permet l'utilisation de la classe par Lunar
			friend class Lunar<Personnage>;
			/// Nom de la classe Lunar
			static const char className[];
			/// Descripteurs de methodes Lunar
			static Lunar<Personnage>::RegType methods[];
			
			/// Pointeur sur le script Lua de l'IA
			lua_State* mIA;
			
			/// Nom du personnage
			std::string mNom;
			/// Direction dans laquelle regarde le personnage
			unsigned int mDirection;
			/// Vitesse de deplacement
			double mVitesse;
			/// Numero du pas
			double mPas;
			/// Le personange est-il visible
			bool mVisible;
			/// Abscisse de la cible
			double mCibleX;
			/// Ordonnee de la cible
			double mCibleY;
			/// Affiche ce que dit le personnage
			Texte* mPhrase;
			
			/// Pointeur sur une cible eventuelle
			GS::Etre* mCible;
			/// Identifiant de la cible a charger le cas echeant
			unsigned int mCibleACharger;
			
			/// Masse d'objets et de ressources transportee
			double mMasseTransportee;
			/// Masse d'objets et de ressources transportable
			double mMasseTransportable;
			/// Quantites des differentes ressoruces
			double* mQuantitesRessources;
			
			/// Batiment dans lequel le personnage se trouve, le cas echeant
			GS::Batiment* mBatimentActuel;
			/// Identifiant du batiment a charger le cas echeant
			unsigned int mBatimentACharger;
			
			/// Objets transportes,  tableau a 4 dimensions (onglet / categorie, page, ligne, colone)
			std::vector < std::vector < std::vector < std::vector <GS::Objet*> > > > mObjetsTransportes;
			
			/// Image contenant les differentes positions du personnage
			sf::Image* mImagePerso;
			/// Afficheur du personnage
			sf::Sprite* mAffichePerso;
	};
}

#endif
